package dsuyasov.recognition.rulesModel;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.DOMException;

import java.util.LinkedList;
import java.util.List;
import java.awt.*;

import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.rulesModel.sequence.Processable;
import dsuyasov.recognition.rulesModel.sequence.AutomataSequence;
import dsuyasov.recognition.rulesModel.sequence.MatrixChangeListener;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.files.XMLTransferable;
import dsuyasov.recognition.files.LoadException;

/**
 * @author Dmitriy Suyasov
 */
public class CellularAutomata extends Processable implements XMLTransferable, MatrixChangeListener {

    public static final String TYPE = "rule";

    public static final String ROOT_RULE_NODE = "rule";
    public static final String TYPE_ATTR_NAME = "type";

    public static final String STRATEGY_RESULT_COLOR = "strategyResultColor";

    private static final String DEFAULT_NAME = "<No Name>";

    private static final String NAME_NODE = "name";
    private static final String DESCRIPTION_NODE = "description";
    private static final String SETTINGS_NODE = "settings";
    private static final String DEFAULT_NODE = "default";
    private static final String RULE_ITEM_NODE = "ruleItem";

    private static final String PRIORITY_ATTR = "priority";
    private static final String STRATEGY_ATTR = "strategy";
    private static final String TYPE_ATTR = "type";
    private static final String DIMENTION_ATTR = "dimention";
    private static final String CONTINUE_CONDITION_ATTR = "continueCondition";
    private static final String COLORS_ACCEPTABLE_ATTR = "colorsAcceptable";
    private static final String RESULT_COLORS_ACCEPTABLE_ATTR = "resultColorsAcceptable";

    private RuleStrategy ruleStrategy;
    private AutomataDimention automataDimention;
    private ContinueCondition continueCondition;
    private TypeOfColors typeOfColors, resultColors;

    private List<RuleItem> ruleItems;
    private RuleItem defaultRuleItem;

    private String name;
    private String description;
    private List<MatrixChangeListener> listeners = new LinkedList<MatrixChangeListener>();

    public CellularAutomata(String name, RuleStrategy ruleStrategy, AutomataDimention automataDimention,
                ContinueCondition continueCondition, TypeOfColors typeOfColors, TypeOfColors resultTypeOfColors) {
        this.ruleStrategy = ruleStrategy;
        this.automataDimention = automataDimention;
        this.continueCondition = continueCondition;
        this.typeOfColors = typeOfColors;
        this.resultColors = resultTypeOfColors;

        this.ruleItems = new LinkedList<RuleItem>();
        this.defaultRuleItem = new StandardRuleItem(this.automataDimention, this.typeOfColors, this.resultColors, this.ruleStrategy);

        this.name = (name == null || name.length() == 0) ? DEFAULT_NAME : name;
        this.description = "";

    }

    public CellularAutomata(Document document) throws LoadException {
        loadRule(XMLUtil.getSingleNode(document, ROOT_RULE_NODE));
    }
    
    public CellularAutomata(Node rootNode) throws LoadException {
        loadRule(rootNode);
    }
    
    public void loadRule(Node rootNode) throws LoadException {
        try {
            this.name = XMLUtil.getNodeValue(XMLUtil.getSingleNode(rootNode, NAME_NODE));
            this.description = XMLUtil.getNodeValue(XMLUtil.getSingleNode(rootNode, DESCRIPTION_NODE));
            this.description = (this.description == null) ? "" : this.description;

            Node settingsNode = XMLUtil.getSingleNode(rootNode, SETTINGS_NODE);
            try {
                automataDimention = AutomataDimention.valueOf( XMLUtil.getRequiredAttribute(settingsNode, DIMENTION_ATTR) );
                ruleStrategy = RuleStrategy.valueOf( XMLUtil.getRequiredAttribute(settingsNode, STRATEGY_ATTR) );
                continueCondition = ContinueCondition.valueOf( XMLUtil.getRequiredAttribute(settingsNode, CONTINUE_CONDITION_ATTR) );
                typeOfColors = TypeOfColors.valueOf( XMLUtil.getRequiredAttribute(settingsNode, COLORS_ACCEPTABLE_ATTR) );
                resultColors = TypeOfColors.valueOf( XMLUtil.getRequiredAttribute(settingsNode, RESULT_COLORS_ACCEPTABLE_ATTR) );
            } catch (IllegalArgumentException e) {
                throw new LoadException("Incorrect dimention, strategy or the type of the rule was set");
            }

            // get default ruleItem
            this.defaultRuleItem = new StandardRuleItem(XMLUtil.getSingleNode(rootNode, DEFAULT_NODE), this.automataDimention);

            // get other ruleItems
            this.ruleItems = new LinkedList<RuleItem>();
            List<Node> ruleItemsNodes = XMLUtil.getNode(rootNode, RULE_ITEM_NODE);
            for (Node ruleItemsNode : ruleItemsNodes) {
                ruleItems.add(new StandardRuleItem(ruleItemsNode, this.automataDimention));
            }
        } catch (DOMException e) {
            throw new LoadException("Problem with DOM document");
        }
    }

    public boolean isSpecialElement() {
        return false;
    }

    public boolean isRule() {
        return true;
    }

    public String getTitle() {
        return name;
    }

    public String getLocalizationTitle() {
        return name;
    }

    public void draw(Graphics g, Rectangle rect, int xTextOffset, int yTextOffset) {
        g.setColor(Color.BLACK);
        g.drawRect(rect.x, rect.y, rect.width, rect.height);
        g.drawString(getTitle(), rect.x + xTextOffset, rect.y + yTextOffset);
    }

    public List<RuleItem> getRuleItems() {
        return ruleItems;
    }

    public RuleItem getDefaultRuleItem() {
        return defaultRuleItem;
    }

    public AutomataDimention getRuleDimention() {
        return automataDimention;
    }

    public ContinueCondition getContinueCondition() {
        return continueCondition;
    }

    public void setContinueCondition(ContinueCondition continueCondition) {
        this.continueCondition = continueCondition;
    }

    public TypeOfColors getColors() {
        return typeOfColors;
    }

    public TypeOfColors getResultColors() {
        return resultColors;
    }

    public void addNewRule() {
        ruleItems.add(new StandardRuleItem(automataDimention, typeOfColors, resultColors, ruleStrategy));
    }

    public void removeRule(int index) {
        ruleItems.remove(index);
    }

    public void ruleInterchangeDown(int first) {
        if (first < ruleItems.size() - 1) {
            RuleItem ruleItem = ruleItems.remove(first);
            ruleItems.add(first + 1, ruleItem);
        }
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public RuleStrategy getRuleStrategy() {
        return ruleStrategy;
    }

    public RuleItem getSelectedRuleItem() {
        Integer selectedIndex = (Integer) Memory.get(Memory.SELECTED_RULE);
        if (selectedIndex > getRuleItems().size()) {
            selectedIndex = 0;
            Memory.add(Memory.SELECTED_RULE, selectedIndex);
        }
        return (selectedIndex == 0) ? getDefaultRuleItem() : getRuleItems().get(selectedIndex - 1);
    }

    public void toXML(Document document, Node rootNode) {
        try {
            XMLUtil.addAttribute(document, rootNode, TYPE_ATTR, TYPE);

            Node nameNode = XMLUtil.addNode(document, rootNode, NAME_NODE);
            XMLUtil.addCDATASection(document, nameNode, getName());

            Node descriptionNode = XMLUtil.addNode(document, rootNode, DESCRIPTION_NODE);
            XMLUtil.addCDATASection(document, descriptionNode, getDescription());

            Node settingsNode = XMLUtil.addNode(document, rootNode, SETTINGS_NODE);
            XMLUtil.addAttribute(document, settingsNode, STRATEGY_ATTR, getRuleStrategy().name());
            XMLUtil.addAttribute(document, settingsNode, DIMENTION_ATTR, getRuleDimention().name());
            XMLUtil.addAttribute(document, settingsNode, CONTINUE_CONDITION_ATTR, getContinueCondition().name());
            XMLUtil.addAttribute(document, settingsNode, COLORS_ACCEPTABLE_ATTR, getColors().name());
            XMLUtil.addAttribute(document, settingsNode, RESULT_COLORS_ACCEPTABLE_ATTR, getResultColors().name());

            Node defaultItem = XMLUtil.addNode(document, rootNode, DEFAULT_NODE);
            defaultRuleItem.toXML(document, defaultItem);

            int iLoop = 0;
            for (RuleItem ruleItem : ruleItems) {
                Node ruleItemNode = XMLUtil.addNode(document, rootNode, RULE_ITEM_NODE);
                XMLUtil.addAttribute(document, ruleItemNode, PRIORITY_ATTR, ""+iLoop);
                ruleItem.toXML(document, ruleItemNode);
                iLoop++;
            }
            
        } catch (Exception e) {
            throw new RuntimeException("Problems while creation of XML", e);
        }
    }

    public void initialize(AutomataSequence automataSequence) {
        this.listeners.clear();
        this.index = 0;
    }

    public void addChangeListener(MatrixChangeListener listener) {
        this.listeners.add(listener);
    }

    public ProcessResult process(MatrixElement[][] source) {
        increceValue = 1;
        // initialization
        if (continueCondition.equals(ContinueCondition.ONCE) || continueCondition.equals(ContinueCondition.WHILE_FIRST_OCCURED)) {
            MatrixElement[][] matrixElements = go(source);
            if (isChanged()) {
                sendChangeRequestToListeners();
            }
            return new ProcessResult(matrixElements);
        }

        MatrixElement[][] destination = go(source);
        index = 0;
        boolean changed = isChanged();
        if (changed) {
            sendChangeRequestToListeners();
        }
        while (isChanged() && index < 100) {
            destination = step(destination);
        }

        return new ProcessResult(destination);
    }

    private MatrixElement[][] step(MatrixElement[][] source) {
        MatrixElement[][] destination = go(source);
        index++;
        if (index == 100) {
            if (MessageUtil.question(Localization.get(Localization.RULE_IN_THE_LOOP_MESSAGE, getName(), String.valueOf(index)))) {
                index = 0;
            }
        }
        return destination;
    }

    private int index;
    private int increceValue;

    public Integer getNext(int ruleIndex) {
        return ruleIndex + increceValue;
    }

    public ProcessResult processStep(MatrixElement[][] source) {
        if (continueCondition.equals(ContinueCondition.ONCE) || continueCondition.equals(ContinueCondition.WHILE_FIRST_OCCURED)) {
            return process(source);
        } else {
            MatrixElement[][] destination = step(source);
            if (isChanged()) {
                sendChangeRequestToListeners();
                increceValue = 0;
            } else {
                increceValue = 1;
            }
            return new ProcessResult(destination);
        }
    }

    private void sendChangeRequestToListeners() {
        for (MatrixChangeListener listener : listeners) {
            listener.change();
        }
    }

    private boolean isChanged() {
        return changes;
    }

    private boolean changes;

    private MatrixElement[][] go(MatrixElement[][] source) {
        Memory.incIncreaser();
        this.changes = false;
        defaultRuleItem.initialize(source, this);
        for (RuleItem ruleItem : ruleItems) {
            ruleItem.initialize(source, this);
        }

        // run
        boolean firstOccuredCondition = continueCondition.equals(ContinueCondition.WHILE_FIRST_OCCURED);
        verybeggining: for (int iLoop1 = 0; iLoop1 < source.length; iLoop1++)
            for (int iLoop2 = 0; iLoop2 < source[0].length; iLoop2++) {
                defaultRuleItem.process(source, iLoop1, iLoop2);
                if (firstOccuredCondition && changes) {
                    break verybeggining;
                }
                for (RuleItem ruleItem : ruleItems) {
                    ruleItem.process(source, iLoop1, iLoop2);
                    if (firstOccuredCondition && changes) {
                        break verybeggining;
                    }
                }
            }

        // finilize
        for (int iLoop1 = 0; iLoop1 < source.length; iLoop1++)
            for (int iLoop2 = 0; iLoop2 < source[0].length; iLoop2++) {
                source[iLoop1][iLoop2].makeActions();
            }

        return source;
    }

    public void change() {
        this.changes = true;
    }
}