package dsuyasov.recognition.rulesModel;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

import java.awt.*;
import java.util.List;
import java.util.LinkedList;

import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.rulesModel.checkers.*;
import dsuyasov.recognition.files.XMLTransferable;
import dsuyasov.recognition.files.LoadException;

/**
 * @author Dmitriy Suyasov
 */
public class RuleCell implements XMLTransferable {

    private static Color[] colors = new Color[10];

    static {
        colors[0] = Color.BLUE;
        colors[1] = Color.RED;
        colors[2] = Color.ORANGE;
        colors[3] = Color.GREEN;
        colors[4] = Color.PINK;
        colors[5] = Color.YELLOW;
        colors[6] = Color.MAGENTA;
        colors[7] = Color.CYAN;
        colors[8] = Color.DARK_GRAY;
        colors[9] = Color.LIGHT_GRAY;
    }

    private static final String COLOR_ACCEPTABLE_ATTR_NAME = "colorAcceptable";
    private static final String SYMBOL_CHECKER_NODE_NAME = "symbolChecker";
    private static final String COLOR_CHECKER_NODE_NAME = "colorChecker";

    private List<LabelChecker> labelCheckers;
    private ColorChecker colorChecker;

    private TypeOfColors colorTypeOf;

    public RuleCell(TypeOfColors colorTypeOf, RuleStrategy strategy) {
        labelCheckers = new LinkedList<LabelChecker>();
        if (strategy.equals(RuleStrategy.NONE)) {
            colorChecker = new FixedColorChecker();
        } else if (strategy.equals(RuleStrategy.DO_NOT_CHANGE) || strategy.equals(RuleStrategy.ALL_TO_COLOR)) {
            colorChecker = new AnyColorChecker();
        } else {
            colorChecker = new AnyColorChecker();
        }
        this.colorTypeOf = colorTypeOf;
    }

    public List<LabelChecker> getLabelCheckers() {
        return labelCheckers;
    }

    public ColorChecker getColorChecker() {
        return colorChecker;
    }

    public void setColorChecker(ColorChecker colorChecker) {
        this.colorChecker = colorChecker;
    }

    public void setLabelCheckers(List<LabelChecker> labelCheckers) {
        this.labelCheckers = labelCheckers;
    }

    public RuleCell(Node node) throws LoadException {
        try {
            colorTypeOf = TypeOfColors.valueOf(XMLUtil.getRequiredAttribute(node, COLOR_ACCEPTABLE_ATTR_NAME));
        } catch (IllegalArgumentException e) {
            throw new LoadException("Incorrect dimention, strategy or the type of the rule was set");
        }
        List<Node> labelCheckerNodes = XMLUtil.getNode(node, SYMBOL_CHECKER_NODE_NAME);
        labelCheckers = new LinkedList<LabelChecker>();
        for (Node labelCheckerNode : labelCheckerNodes) {
            labelCheckers.add(LabelChecker.get(labelCheckerNode));
        }
        colorChecker = ColorChecker.get(XMLUtil.getSingleNode(node, COLOR_CHECKER_NODE_NAME));
    }
    
    public boolean process(MatrixElement[][] source, int x, int y) {
        boolean result = colorChecker.check(source, x, y);
        int index = 0;
        while (result && index < labelCheckers.size())
            result = labelCheckers.get(index++).check(source, x, y);
        return result;
    }

    public TypeOfColors getColorAcceptable() {
        return colorTypeOf;
    }

    public void drawCell(Graphics g, Rectangle rect) {
        getColorChecker().drawCell(g, rect);
        if (labelCheckers.size() > 0) {
            g.setColor(Color.WHITE);
            g.fillRect(rect.x + 2, rect.y + 2, rect.width - 4, rect.height / 3);
            int step = (rect.width - 6) / labelCheckers.size();
            int index = 0;
            for (LabelChecker labelChecker : labelCheckers) {
                g.setColor(labelChecker.getColor());
                g.fillRect(rect.x + 3 + step * index++, rect.y + 3, step, rect.height / 3 - 2);
            }
        }
    }

    public void toXML(Document doc, Node rootElement) {
        XMLUtil.addAttribute(doc, rootElement, COLOR_ACCEPTABLE_ATTR_NAME, colorTypeOf.name());
        for (LabelChecker labelChecker : labelCheckers) {
            labelChecker.toXML(doc, XMLUtil.addNode(doc, rootElement, SYMBOL_CHECKER_NODE_NAME));
        }
        colorChecker.toXML(doc, XMLUtil.addNode(doc, rootElement, COLOR_CHECKER_NODE_NAME));
    }
}
