package dsuyasov.recognition.rulesModel;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

import java.util.Map;
import java.util.HashMap;
import java.util.List;

import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.rulesModel.sequence.MatrixChangeListener;
import dsuyasov.recognition.files.LoadException;

/**
 * @author Dmitriy Suyasov
 */
public class StandardRuleItem extends RuleItem {

    private static final String RESULT_NODE = "result";
    private static final String CELL_NODE = "cell";

    private static final String LINE_ATTR = "line";
    private static final String COLUMN_ATTR = "column";

    // Number of line and than number of column
    public Map<Integer, Map<Integer, RuleCell>> cells;

    public RuleCellResult result;

    public StandardRuleItem(AutomataDimention dimention, TypeOfColors typeOfColors, TypeOfColors resultTypeOfColors, RuleStrategy ruleStrategy) {
        super(dimention.getDimention());
        cells = new HashMap<Integer, Map<Integer, RuleCell>>();
        for (int iLoop = -this.dimention; iLoop <= this.dimention; iLoop++) {
            Map<Integer, RuleCell> cellsLine = new HashMap<Integer, RuleCell>();
            for (int iLoop2 = -this.dimention; iLoop2 <= this.dimention; iLoop2++) {
                cellsLine.put(iLoop2, new RuleCell(typeOfColors, ruleStrategy));
            }
            cells.put(iLoop, cellsLine);
        }
        result = new RuleCellResult(resultTypeOfColors, ruleStrategy);
    }

    public StandardRuleItem(Node node, AutomataDimention dimention) throws LoadException {
        super(dimention.getDimention());

        //get all other cells
        cells = new HashMap<Integer, Map<Integer, RuleCell>>();

        List<Node> cellNodes = XMLUtil.getNode(node, CELL_NODE);
        for (Node cellNode : cellNodes) {
            try {
                int line = XMLUtil.getIntAttribute(cellNode, LINE_ATTR);
                int column = XMLUtil.getIntAttribute(cellNode, COLUMN_ATTR);
                setNewCell(line, column, new RuleCell(cellNode));
            } catch (NumberFormatException nfe) {
                throw new LoadException("Wrong line or column of the cell cpecified", nfe);
            }
        }
        for (int iLoop = -this.dimention; iLoop <= this.dimention; iLoop++) {
            Map<Integer, RuleCell> theline = cells.get(iLoop);
            if (theline == null)
                throw new LoadException("Line " + iLoop + " was not specified");
            for (int iLoop2 = -this.dimention; iLoop2 <= this.dimention; iLoop2++) {
                RuleCell cell = theline.get(iLoop2);
                if (cell == null)
                    throw new LoadException("Cell (" + iLoop + ", " + iLoop2 + ") was not specified");
            }
        }

        // get result
        result = new RuleCellResult(XMLUtil.getSingleNode(node, RESULT_NODE), cells.get(0).get(0));
    }

    private void setNewCell(int line, int column, RuleCell cell) {
        Map<Integer, RuleCell> theline = cells.get(line);
        if (theline == null) {
            theline = new HashMap<Integer, RuleCell>();
            cells.put(line, theline);
        }
        theline.put(column, cell);
    }

    public RuleCell getCell(int line, int column) {
        if (Math.abs(line) > dimention || Math.abs(column) > dimention)
            throw new RuntimeException("Someone try to get cell [" + line + ", " + column + "] but dimention is " +
                    dimention);
        return cells.get(line).get(column);
    }

    public RuleCellResult getResult() {
        return result;
    }

    public void changeRule(int line, int column, RuleCell cell) {
        if (Math.abs(line) > dimention || Math.abs(column) > dimention)
            throw new RuntimeException("Someone try to change cell [" + line + ", " + column + "] but dimention is " +
                    dimention);
        cells.get(line).put(column, cell);
    }

    public void changeResult(RuleCellResult cell) {
        this.result = cell;
    }

    public void toXML(Document doc, Node rootElement) {
        result.toXML(doc, XMLUtil.addNode(doc, rootElement, RESULT_NODE));

        for (Integer lineNumber : cells.keySet()) {
            Map<Integer,RuleCell> line = cells.get(lineNumber);
            for (Integer columnNumber : line.keySet()) {
                Node cellNode = XMLUtil.addNode(doc, rootElement, CELL_NODE);
                XMLUtil.addAttribute(doc, cellNode, LINE_ATTR, lineNumber.toString());
                XMLUtil.addAttribute(doc, cellNode, COLUMN_ATTR, columnNumber.toString());

                line.get(columnNumber).toXML(doc, cellNode);
            }
        }
    }

    public void initialize(MatrixElement[][] source, MatrixChangeListener listener) {
        result.initialize(this, source, listener);
    }

    public void process(MatrixElement[][] source, int x, int y) {
        boolean allow = cells.get(0).get(0).process(source, x, y);
        if (allow && getDimention() > 0) {
            allow = cells.get(1).get(0).process(source, x + 1, y);
            allow = allow && cells.get(0).get(1).process(source, x, y + 1);
            allow = allow && cells.get(-1).get(0).process(source, x - 1, y);
            allow = allow && cells.get(0).get(-1).process(source, x, y - 1);

            allow = allow && cells.get(1).get(1).process(source, x + 1, y + 1);
            allow = allow && cells.get(1).get(-1).process(source, x + 1, y - 1);
            allow = allow && cells.get(-1).get(-1).process(source, x - 1, y - 1);
            allow = allow && cells.get(-1).get(1).process(source, x - 1, y + 1);
        }
        if (allow && getDimention() > 1) {
            allow = cells.get(2).get(0).process(source, x, y);
            allow = allow && cells.get(0).get(2).process(source, x, y + 2);
            allow = allow && cells.get(-2).get(0).process(source, x - 2, y);
            allow = allow && cells.get(0).get(-2).process(source, x, y - 2);

            allow = allow && cells.get(2).get(1).process(source, x + 2, y + 1);
            allow = allow && cells.get(2).get(-1).process(source, x + 2, y - 1);
            allow = allow && cells.get(1).get(-2).process(source, x + 1, y - 2);
            allow = allow && cells.get(-1).get(-2).process(source, x - 1, y - 2);
            allow = allow && cells.get(-2).get(-1).process(source, x - 2, y - 1);
            allow = allow && cells.get(-2).get(1).process(source, x - 2, y + 1);
            allow = allow && cells.get(-1).get(2).process(source, x - 1, y + 2);
            allow = allow && cells.get(1).get(2).process(source, x + 1, y + 2);

            allow = allow && cells.get(2).get(2).process(source, x + 2, y + 2);
            allow = allow && cells.get(2).get(-2).process(source, x + 2, y - 2);
            allow = allow && cells.get(-2).get(-2).process(source, x - 2, y - 2);
            allow = allow && cells.get(-2).get(2).process(source, x - 2, y + 2);
        }
        if (allow)
            result.process(x, y);
    }

}