package dsuyasov.recognition.rulesModel.labels;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.files.LoadException;

/**
 * Developed by Suyasov Dmitriy
 */
public class PredefinedLabels extends Labels {

    public static final String PREDEFINE_SYMBOLS_TITLE = "Standard Symbol";

    public static final int POINT = 0;
    public static final int LINE = 1;
    public static final int CIRCLE = 2;
    public static final int SQUARE = 3;
    public static final int PLUS = 4;
    public static final int MINUS = 5;
    public static final int SLASH = 6;
    public static final int PLUS_IN_CIRCLE = 7;
    public static final int MINUS_IN_CIRCLE = 8;
    public static final int POINT_IN_CIRCLE = 9;

    public static final String POINT_SYMBOL = "Point";
    public static final String LINE_SYMBOL = "Line";
    public static final String CIRCLE_SYMBOL = "Circle";
    public static final String SQUARE_SYMBOL = "Square";
    public static final String PLUS_SYMBOL = "Plus";
    public static final String MINUS_SYMBOL = "Minus";
    public static final String SLASH_SYMBOL = "Slash";
    public static final String PLUS_IN_CIRCLE_SYMBOL = "Plus in Circle";
    public static final String MINUS_IN_CIRCLE_SYMBOL = "Minus in Circle";
    public static final String POINT_IN_CIRCLE_SYMBOL = "Point in Circle";

    public static final String[] titles = {POINT_SYMBOL, LINE_SYMBOL, CIRCLE_SYMBOL, SQUARE_SYMBOL, PLUS_SYMBOL,
            MINUS_SYMBOL, SLASH_SYMBOL, PLUS_IN_CIRCLE_SYMBOL, MINUS_IN_CIRCLE_SYMBOL, POINT_IN_CIRCLE_SYMBOL};

    private static final String TYPE_NUMBER_NODE_NAME = "typeNumber";
    private static final String VALUE_ATTR_NAME = "value";

    private static final LabelDrawer[] drawers = {new PointDrawer(), new LineDrawer(), new CircleDrawer(),
            new SquareDrawer(), new PlusDrawer(), new MinusDrawer(), new SlashDrawer(), new PlusInCircleDrawer(),
            new MinusInCircleDrawer(), new PointInCircleDrawer()};

    private static final Color[] colors = {Color.RED, Color.BLUE, Color.GREEN, Color.MAGENTA, Color.ORANGE, Color.PINK,
            Color.CYAN, Color.LIGHT_GRAY, Color.YELLOW, Color.DARK_GRAY};

    private int type;

    private JPanel panel;

    public PredefinedLabels(int type) {
        if (type < 0 || type >= titles.length)
            throw new RuntimeException("Wrong type of PredefinedLabels");
        this.type = type;
    }

    public PredefinedLabels() {
        this.type = 0;
    }

    public PredefinedLabels(Node node) throws LoadException {
        this.type = XMLUtil.getIntAttribute(XMLUtil.getSingleNode(node, TYPE_NUMBER_NODE_NAME), VALUE_ATTR_NAME);
    }

    public Color getColor() {
        return colors[type];
    }

    public String getTypeTitle() {
        return titles[this.type];
    }

    public String getTitle() {
        return PREDEFINE_SYMBOLS_TITLE;
    }

    public String getLocalizationTitle() {
        return Localization.get(Localization.PREDEFINE_SYMBOL_TITLE);
    }

    public int getType() {
        return type;
    }

    public void draw(Graphics g, int x, int y, int width, int height) {
        g.setColor(colors[type]);
        g.fillRect(x + width / 4, y + height / 4, width / 2, height / 2);
    }

    public void drawDialog(JComponent parent) {
        if (this.panel == null || !this.panel.isAncestorOf(parent)) {
            this.panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
            parent.add(this.panel);
        }
        panel.removeAll();
        panel.add(new JLabel(Localization.get(Localization.PREDEFINE_SYMBOL_TITLE)));
        for (int iLoop = 0; iLoop < drawers.length; iLoop++) {
            JPanel button = new LabelButton(iLoop, this, parent);
            panel.add(button);
        }
        parent.repaint();
    }

    public boolean verify() {
        return this.type >= 0 && this.type < drawers.length;
    }

    public void changeType(int type, JComponent parent) {
        this.type = type;
        drawDialog(parent);
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        XMLUtil.addAttribute(doc, XMLUtil.addNode(doc, rootElement, TYPE_NUMBER_NODE_NAME), VALUE_ATTR_NAME, String.valueOf(getType()));
    }

    private class LabelButton extends JPanel implements MouseListener {

        private int type;
        private final PredefinedLabels predefinedLabels;
        private JComponent parent;

        public LabelButton(int type, PredefinedLabels predefinedLabels, JComponent parent) {
            setPreferredSize(new Dimension(30, 30));
            setBorder(BorderFactory.createLineBorder(Color.BLACK));
            this.type = type;
            this.parent = parent;
            this.predefinedLabels = predefinedLabels;
            addMouseListener(this);
        }

        public void paint(Graphics g) {
            super.paint(g);
            g.setColor(colors[type]);
            g.fillRect(this.getWidth() / 4, this.getHeight() / 4, this.getWidth() / 2, this.getHeight() / 2);
            if (predefinedLabels.getType() == type) {
                g.setColor(Color.BLACK);
                g.fillRect(0, 0, this.getWidth() / 5, this.getHeight() / 5);
                g.fillRect(this.getWidth() * 4 / 5, 0, this.getWidth() / 5, this.getHeight() / 5);
                g.fillRect(0, this.getHeight() * 4 / 5, this.getWidth() / 5, this.getHeight() / 5);
                g.fillRect(this.getWidth() * 4 / 5, this.getHeight() * 4 / 5, this.getWidth() / 5, this.getHeight() / 5);
                g.fillRect(this.getWidth() * 4 / 5, this.getHeight() * 4 / 5, this.getWidth() / 5, this.getHeight() / 5);
            }
        }

        public void mouseClicked(MouseEvent e) {
            predefinedLabels.changeType(this.type, parent);
        }

        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
    }

    private interface LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height);
    }

    private static class PointDrawer implements LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height) {
            g.fillRect(x + width / 4, y + height / 4, 2, 2);
        }
    }

    private static class LineDrawer implements LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height) {
            g.fillRect(x + width / 4, y + 2, 2, height / 4);
        }
    }

    private static class CircleDrawer implements LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height) {
            g.drawOval(x + 2, y + 2, width / 4, height / 4);
        }
    }

    private static class SquareDrawer implements LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height) {
            g.fillRect(x + 2, y + 2, width / 4, height / 4);
        }
    }

    private static class PlusDrawer implements LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height) {
            g.fillRect(x + 2, y + height / 4, width / 4, 2);
            g.fillRect(x + width / 4, y + 2, 2, height / 4);
        }
    }

    private static class MinusDrawer implements LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height) {
            g.fillRect(x + 2, y + height / 4, width / 4, 2);
        }
    }

    private static class SlashDrawer implements LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height) {
            g.drawLine(x + width / 4, y + 2, x + 2, y + height / 4);
        }
    }

    private static class PlusInCircleDrawer implements LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height) {
            g.drawOval(x + 2, y + 2, width / 4, height / 4);
            g.fillRect(x + 2, y + 1 + height / 8, width / 4, 2);
            g.fillRect(x + 2 + width / 8, y + 2, 1, height / 4);
        }
    }

    private static class MinusInCircleDrawer implements LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height) {
            g.drawOval(x + 2, y + 2, width / 4, height / 4);
            g.fillRect(x + 2, y + 2 + height / 8, width / 4, 1);
        }
    }

    private static class PointInCircleDrawer implements LabelDrawer {
        public void draw(Graphics g, int x, int y, int width, int height) {
            g.drawOval(x + 2, y + 2, width / 4, height / 4);
            g.fillRect(x + 2 + width / 8, y + 2 + height / 8, 1, 1);
        }
    }

    public boolean equals(Object obj) {
        return obj instanceof PredefinedLabels && this.getType() == ((PredefinedLabels) obj).getType();
    }
}
